@@ -204,8 +204,17 @@
         this.$icon = 'thumb_down';
       }
       else if (this.type == 'additional') {
+        this.$icon = 'folder';
+      }
+      else if (this.type == 'shared') {
         this.$icon = 'folder_shared';
       }
+      else if (this.type == 'otherUsers') {
+        this.$icon = 'folder_shared';
+      }
+      else if (this.type == 'dropbox') {
+        this.$icon = 'drive_folder_upload';
+      }
       else {
         this.$isSpecial = false;
         this.$icon = 'folder';
@@ -394,6 +403,18 @@
       options.syncToken = this.$syncToken;
     }
 
+    if (this.$unseenOnly)
+      options.unseenOnly = 1;
+
+    if (this.$flaggedOnly)
+      options.flaggedOnly = 1;
+
+    var labels = _.filter(_.keys(this.$filteredLabels), function (k) {
+      return !!_this.$filteredLabels[k];
+    });
+    if (labels.length)
+      options.labels = labels;
+
     // Restart the refresh timer, if needed
     if (!Mailbox.$virtualMode) {
       var refreshViewCheck = Mailbox.$Preferences.defaults.SOGoRefreshViewCheck;
@@ -502,6 +523,16 @@
     return this.flags.indexOf('noselect') >= 0;
   };
 
+  /**
+   * @function isWritable
+   * @memberof Mailbox.prototype
+   * @desc Checks the user can write to the mailbox
+   * @returns true if messages can be inserted
+   */
+  Mailbox.prototype.isWritable = function() {
+    return this.flags.indexOf('noselect') < 0 || this.type == 'dropbox';
+  };
+
   /**
    * @function getClassName
    * @memberof Mailbox.prototype
@@ -628,17 +659,18 @@
   };
 
   /**
-   * @function $emptyTrash
+   * @function $empty
    * @memberof Mailbox.prototype
    * @desc Empty the Trash folder.
    * @returns a promise of the HTTP operation
    */
-  Mailbox.prototype.$emptyTrash = function() {
-    var _this = this;
+  Mailbox.prototype.$empty = function() {
+    var _this = this,
+        action = 'empty' + this.type[0].capitalize() + this.type.substring(1);
 
-    return Mailbox.$$resource.post(this.id, 'emptyTrash').then(function(data) {
+    return Mailbox.$$resource.post(this.id, action).then(function(data) {
       // Remove all messages from the mailbox
-      _this.$messages = [];
+      _this.$messages = _this.$visibleMessages = [];
       _this.uidsMap = {};
       _this.unseenCount = 0;
 
@@ -669,6 +701,30 @@
     });
   };
 
+  /**
+   * @function getLabels
+   * @memberof Mailbox.prototype
+   * @desc Fetch the list of labels associated to the mailbox. Use the cached value if available.
+   * @returns a promise of the HTTP operation
+   */
+  Mailbox.prototype.getLabels = function(options) {
+    var _this = this;
+
+    if (this.$labels && !(options && options.reload))
+      return Mailbox.$q.when(this.$labels);
+
+    if (angular.isUndefined(this.$filteredLabels))
+      this.$filteredLabels = {};
+    return Mailbox.$$resource.fetch(this.id, 'labels').then(function(data) {
+      _this.$labels = data;
+      return _this.$labels;
+    });
+  };
+
+  Mailbox.prototype.filteredByLabel = function() {
+    return _.includes(this.$filteredLabels, 1);
+  };
+
   /**
    * @function $flagMessages
    * @memberof Mailbox.prototype
@@ -789,9 +845,6 @@
           data = { uids: currentUids };
       if (options) angular.extend(data, options);
       return Mailbox.$$resource.post(_this.id, 'batchDelete', data).then(function(data) {
-        if (data.unseenCount) {
-          _this.unseenCount = data.unseenCount;
-        }
         if (end < uids.length) {
           _this.$_deleteMessages(currentUids);
           return _deleteMessages(end, Math.min(end + batchSize, uids.length));
@@ -800,6 +853,9 @@
           // Last API call; update inbox quota
           if (data.quotas)
             _this.$account.updateQuota(data.quotas);
+          if (angular.isDefined(data.unseenCount))
+            _this.unseenCount = data.unseenCount;
+
           return _this.$_deleteMessages(currentUids);
         }
       });
@@ -855,7 +911,7 @@
     uids = _.map(messages, 'uid');
     return Mailbox.$$resource.post(this.id, 'moveMessages', {uids: uids, folder: folder})
       .then(function(data) {
-        if (data.unseenCount) {
+        if (angular.isDefined(data.unseenCount)) {
           _this.unseenCount = data.unseenCount;
         }
         _this.$selectedMessages = []; // reset selection
@@ -863,20 +919,6 @@
       });
   };
 
-  /**
-   * @function $reset
-   * @memberof Mailbox.prototype
-   * @desc Reset the original state the mailbox's data.
-   */
-  Mailbox.prototype.$reset = function() {
-    var _this = this;
-    angular.forEach(this.$shadowData, function(value, key) {
-      delete _this[key];
-    });
-    angular.extend(this, this.$shadowData);
-    this.$shadowData = this.$omit();
-  };
-
   /**
    * @function $move
    * @memberof Mailbox.prototype
@@ -924,14 +966,41 @@
     return this.$account.$newMailbox(path, name);
   };
 
+  /**
+   * @function $reset
+   * @memberof Mailbox.prototype
+   * @desc Reset the original state the mailbox's data.
+   */
+  Mailbox.prototype.$reset = function(options) {
+    var _this = this;
+    angular.forEach(this.$shadowData, function(value, key) {
+      delete _this[key];
+    });
+    angular.extend(this, this.$shadowData);
+    this.$shadowData = this.$omit();
+    if (options && options.filter) {
+      this.$messages = [];
+      this.$visibleMessages = [];
+      delete this.$syncToken;
+    }
+  };
+
   /**
    * @function $omit
    * @memberof Mailbox.prototype
    * @desc Return a sanitized object used to send to the server.
    * @return an object literal copy of the Mailbox instance
    */
-  Mailbox.prototype.$omit = function() {
-    var mailbox = {};
+  Mailbox.prototype.$omit = function(deep) {
+    var mailbox = {},
+        _visit = function(children) {
+          var childrenArray = [];
+          _.forEach(children, function(o) {
+            childrenArray.push(o.$omit(deep));
+          });
+          return childrenArray;
+        };
+
     angular.forEach(this, function(value, key) {
       if (key != 'constructor' &&
           key != 'children' &&
@@ -942,6 +1011,9 @@
         mailbox[key] = value;
       }
     });
+    if (deep && this.children) {
+      mailbox.children = _visit(this.children);
+    }
     return mailbox;
   };
 
@@ -1017,7 +1089,7 @@
             }
           }
         }
-        if (data.unseenCount) {
+        if (angular.isDefined(data.unseenCount)) {
           _this.unseenCount = data.unseenCount;
         }
 
@@ -1025,6 +1097,7 @@
           // Initialization phase, we received complete list of UIDs
           Mailbox.$log.debug('unwrapping ' + data.uids.length + ' messages');
 
+          hasNewMessages = true;
           _this.init(data);
 
           // First entry of 'uids' are keys when threaded view is enabled
