@@ -255,11 +255,13 @@
     if (Mailbox.$virtualMode)
       return [];
 
-    if (stateMailbox)
+    if (stateMailbox) {
+      stateMailbox.$unselectMessages();
       promise = stateMailbox.$filter().catch(function() {
         // Mailbox not found
         return $q.reject('Mailbox not found');
       });
+    }
     else
       promise = $q.reject("Mailbox doesn't exist");
 
@@ -315,7 +317,7 @@
       return messageObject.uid == parseInt($stateParams.messageId);
     });
 
-    if (message) {
+    if (message && message.$reload) {
       return message.$reload({useCache: true});
     }
     else {
@@ -329,7 +331,7 @@
    */
   onEnterMessage.$inject = ['$stateParams', 'stateMailbox'];
   function onEnterMessage($stateParams, stateMailbox) {
-    stateMailbox.selectedMessage = parseInt($stateParams.messageId);
+    stateMailbox.$selectedMessage = parseInt($stateParams.messageId);
   }
 
   /**
@@ -337,7 +339,7 @@
    */
   onExitMessage.$inject = ['stateMailbox'];
   function onExitMessage(stateMailbox) {
-    delete stateMailbox.selectedMessage;
+    delete stateMailbox.$selectedMessage;
   }
 
   /**
