@@ -26,6 +26,17 @@
         this.init(futureMessageData);
       }
       this.uid = parseInt(futureMessageData.uid);
+      this.selected = !!futureMessageData.selected;
+      this.level = parseInt(futureMessageData.level);
+      this.first = parseInt(futureMessageData.first) === 1;
+      this.flags = [];
+      if (this.first) {
+        this.threadCount = parseInt(futureMessageData.count);
+        this.collapsed = (futureMessageData.collapsed === true);
+      }
+      else if (!isNaN(this.level) && this.level >= 0) {
+        this.threadMember = true;
+      }
     }
     else {
       // The promise will be unwrapped first
@@ -52,6 +63,8 @@
     // Initialize tags form user's defaults
     if (Preferences.defaults.SOGoMailLabelsColors) {
       Message.$tags = Preferences.defaults.SOGoMailLabelsColors;
+    } else {
+      Message.$tags = {};
     }
     if (Preferences.defaults.SOGoMailDisplayRemoteInlineImages &&
         Preferences.defaults.SOGoMailDisplayRemoteInlineImages == 'always') {
@@ -119,6 +132,8 @@
         _this.flags.splice(i, 1,'_' + flag);
       }
     });
+    // isread will be undefined when composing a new message -- assume unseen flag is not set.
+    this.isread = angular.isDefined(this.isread) ? !!this.isread : true;
   };
 
   /**
@@ -331,17 +346,40 @@
               };
             }
             else if (part.type == 'UIxMailPartEncryptedViewer') {
-              _this.encrypted = {
-                valid: part.valid
-              };
-              if (part.valid)
-                _this.encrypted.message = l("This message is encrypted");
-              else
-                _this.encrypted.message = l("This message can't be decrypted. Please make sure you have uploaded your S/MIME certificate from the mail preferences module.");
+              if (part.encrypted) {
+                _this.encrypted = {
+                  valid: part.decrypted
+                };
+                if (part.decrypted)
+                  _this.encrypted.message = l("This message is encrypted");
+                else
+                  _this.encrypted.message = l("This message can't be decrypted. Please make sure you have uploaded your S/MIME certificate from the mail preferences module.");
+              }
+              if (part.opaqueSigned) {
+                _this.signed = {
+                  valid: part.valid,
+                  certificate: part.certificates[part.certificates.length - 1],
+                  message: part.message
+                };
+              }
             }
-            _.forEach(part.content, function(mixedPart) {
-              _visit(mixedPart);
+            var winmail = _.find(part.content, function(mixedPart) {
+              // Ignore empty content -- that could mean a decoding error server-side.
+              return mixedPart.type == 'UIxMailPartTnefViewer' && mixedPart.content.length > 0;
             });
+
+            if (winmail && !_.find(part.content, function(mixedPart) {
+              return mixedPart.type == 'UIxMailPartAlternativeViewer';
+            })) {
+              // If there's no alternate part in the message, show the winmail.dat attachment only.
+              // Otherwise, show all parts.
+              _visit(winmail);
+            }
+            else {
+              _.forEach(part.content, function(mixedPart) {
+                _visit(mixedPart);
+              });
+            }
           }
           else {
             if (angular.isUndefined(part.safeContent)) {
@@ -425,7 +463,7 @@
       return Message.$$resource.fetch(_this.$absolutePath({asDraft: true}), 'edit').then(function(data) {
         // Try to match a known account identity from the specified "from" address
         var identity = _.find(_this.$mailbox.$account.identities, function(identity) {
-          return data.from.toLowerCase().indexOf(identity.email) !== -1;
+          return data.from && data.from.toLowerCase().indexOf(identity.email) !== -1;
         });
         if (identity)
           data.from = identity.full;
@@ -460,7 +498,18 @@
    * @returns a promise of the HTTP operation
    */
   Message.prototype.addTag = function(tag) {
-    return this.$addOrRemoveTag('add', tag);
+    var _this = this,
+        _tag = tag.replace(/^_\$/, '$');
+    return this.$mailbox.getLabels().then(function(labels) {
+      var reload = !_.find(labels, function(label) {
+        return label.imapName == _tag;
+      });
+      return _this.$addOrRemoveTag('add', tag).then(function() {
+        if (reload)
+          // Update the list of labels for the mailbox
+          _this.$mailbox.getLabels({reload: true});
+      });
+    });
   };
 
   /**
@@ -493,6 +542,31 @@
       return Message.$$resource.post(this.$mailbox.$id(), 'addOrRemoveLabel', data);
   };
 
+  /**
+   * @function toggleRead
+   * @memberof Message.prototype
+   * @desc Toggle message unseen status
+   * @returns a promise of the HTTP operation
+   */
+  Message.prototype.toggleRead = function() {
+    var _this = this;
+
+    if (this.isread)
+      return Message.$$resource.fetch(this.$absolutePath(), 'markMessageUnread').then(function() {
+        Message.$timeout(function() {
+          _this.isread = false;
+          _this.$mailbox.unseenCount++;
+        });
+      });
+    else
+      return Message.$$resource.fetch(this.$absolutePath(), 'markMessageRead').then(function() {
+        Message.$timeout(function() {
+          _this.isread = true;
+          _this.$mailbox.unseenCount--;
+        });
+      });
+  };
+
   /**
    * @function $imipAction
    * @memberof Message.prototype
@@ -520,6 +594,22 @@
     return Message.$$resource.post(this.$absolutePath(), 'sendMDN');
   };
 
+  /**
+   * @function hasAttachments
+   * @memberof Message.prototype
+   * @returns true if there's one ore more attached files
+   */
+  Message.prototype.hasAttachments = function(content) {
+    var _this = this;
+
+    return !!_.find(content || this.parts.content, function(part) {
+      if (angular.isArray(part.content)) {
+        return _this.hasAttachments(part.content);
+      }
+      return part.type == 'UIxMailPartLinkViewer' || part.type == 'UIxMailPartImageViewer';
+    });
+  };
+
   /**
    * @function $deleteAttachment
    * @memberof Message.prototype
@@ -529,7 +619,7 @@
   Message.prototype.$deleteAttachment = function(filename) {
     var data = { 'filename': filename };
     var _this = this;
-    Message.$$resource.fetch(this.$absolutePath({asDraft: true}), 'deleteAttachment', data).then(function(data) {
+    return Message.$$resource.fetch(this.$absolutePath({asDraft: true}), 'deleteAttachment', data).then(function() {
       Message.$timeout(function() {
         _this.editable.attachmentAttrs = _.filter(_this.editable.attachmentAttrs, function(attachment) {
           return attachment.filename != filename;
@@ -539,7 +629,7 @@
   };
 
   /**
-   * @function $markAsFlaggedOrUnflagged
+   * @function toggleFlag
    * @memberof Message.prototype
    * @desc Add or remove a the \\Flagged flag on the current message.
    * @returns a promise of the HTTP operation
@@ -558,6 +648,28 @@
     });
   };
 
+  /**
+   * @function toggleThread
+   * @memberof Message.prototype
+   * @desc Collapse or expand mail thread
+   * @returns a promise of the HTTP operation
+   */
+  Message.prototype.toggleThread = function() {
+    var _this = this,
+        action = 'markMessageCollapse';
+
+    if (this.collapsed)
+      action = 'markMessageUncollapse';
+
+    this.collapsed = !this.collapsed;
+    _this.$mailbox.updateVisibleMessages();
+
+    return Message.$$resource.post(this.$absolutePath(), action).catch(function() {
+      this.collapsed = !this.collapsed;
+      _this.$mailbox.updateVisibleMessages();
+    });
+  };
+
   /**
    * @function $isLoading
    * @memberof Message.prototype
@@ -579,13 +691,16 @@
     var _this = this, futureMessageData;
 
     if (options && options.useCache && this.$futureMessageData) {
+      // The message has already been fetched.
       if (!this.isread) {
-        Message.$$resource.fetch(this.$absolutePath(), 'markMessageRead').then(function() {
-          Message.$timeout(function() {
-            _this.isread = true;
-            _this.$mailbox.unseenCount--;
-          });
-        });
+        if (Message.$Preferences.defaults.SOGoMailAutoMarkAsReadDelay > -1)
+          // Automatically mark message as read
+          _this.$markAsReadPromise = Message.$timeout(function() {
+            Message.$$resource.fetch(_this.$absolutePath(), 'markMessageRead').then(function() {
+              _this.isread = true;
+              _this.$mailbox.unseenCount--;
+            });
+          }, Message.$Preferences.defaults.SOGoMailAutoMarkAsReadDelay * 1000);
       }
       return this;
     }
@@ -692,6 +807,18 @@
           if (accountDefaults.security.alwaysEncrypt)
             data.encrypt = true;
         }
+        if (data.isHTML) {
+          // Sanitize HTML replies to properly display quoted content in CKEditor.
+          // Don't use the DOM to avoid triggering any event.
+          var html = data.text;
+          html = html.replace(/<\/?html[^>]*>/g, '');
+          html = html.replace(/<\/?body[^>]*>/g, '');
+          html = html.replace(/<meta[^>]*>.*<\/meta>/g, '');
+          html = html.replace(/<link[^>]*>.*<\/link>/g, '');
+          html = html.replace(/<base[^>]*>.*<\/base>/g, '');
+          html = html.replace(/<title[^>]*>.*<\/title>/g, '');
+          data.text = html;
+        }
         angular.extend(message.editable, data);
 
         // We keep a reference to our original message in order to update the flags
@@ -709,7 +836,7 @@
    */
   Message.prototype.$save = function() {
     var _this = this,
-        data = this.editable;
+        data = this.$omit();
 
     Message.$log.debug('save = ' + JSON.stringify(data, undefined, 2));
 
@@ -729,7 +856,7 @@
    */
   Message.prototype.$send = function() {
     var _this = this,
-        data = angular.copy(this.editable);
+        data = this.$omit();
 
     Message.$log.debug('send = ' + JSON.stringify(data, undefined, 2));
 
@@ -768,7 +895,18 @@
     // Resolve and expose the promise
     this.$futureMessageData = futureMessageData.then(function(data) {
       // Calling $timeout will force Angular to refresh the view
-      if (_this.isread === 0) {
+      if (!data.isRead) {
+        if (Message.$Preferences.defaults.SOGoMailAutoMarkAsReadDelay > -1)
+          // Automatically mark message as read
+          _this.$markAsReadPromise = Message.$timeout(function() {
+            Message.$$resource.fetch(_this.$absolutePath(), 'markMessageRead').then(function() {
+              _this.isread = true;
+              _this.$mailbox.unseenCount--;
+            });
+          }, Message.$Preferences.defaults.SOGoMailAutoMarkAsReadDelay * 1000);
+      }
+      else if (!_this.isread) {
+        // Message as already been marked read on the server
         _this.isread = true;
         _this.$mailbox.unseenCount--;
       }
@@ -791,9 +929,15 @@
    */
   Message.prototype.$omit = function(options) {
     var message = {},
-        privateAttributes = options && options.privateAttributes;
-    angular.forEach(this, function(value, key) {
-      if (key != 'constructor' && key[0] != '$' || privateAttributes) {
+        privateAttributes = options && options.privateAttributes,
+        source = privateAttributes ? this : this.editable;
+    angular.forEach(source, function(value, key) {
+      if (_.includes(['to', 'cc', 'bcc'], key) && !privateAttributes) {
+        message[key] = _.map(value, function (addr) {
+          return addr.toString();
+        });
+      }
+      else if (key != 'constructor' && key[0] != '$' || privateAttributes) {
         message[key] = value;
       }
     });
@@ -802,12 +946,12 @@
   };
 
   /**
-   * @function download
+   * @function downloadArchive
    * @memberof Message.prototype
-   * @desc Download the current message
+   * @desc Download the current message as a zip archive
    * @returns a promise of the HTTP operation
    */
-  Message.prototype.download = function() {
+  Message.prototype.downloadArchive = function() {
     var data, options;
 
     data = { uids: [this.uid] };
@@ -817,12 +961,25 @@
   };
 
   /**
-   * @function downloadAttachments
+   * @function download
    * @memberof Message.prototype
-   * @desc Download an archive of all attachments
+   * @desc Download the current message as a eml file
    * @returns a promise of the HTTP operation
    */
-  Message.prototype.downloadAttachments = function() {
+  Message.prototype.download = function() {
+    var options;
+
+    options = { filename: this.subject + '.eml' };
+    return Message.$$resource.download(this.$absolutePath(), 'export', undefined, options);
+  };
+
+  /**
+   * @function downloadAttachments
+   * @memberof Message.prototype
+   * @desc Download a zip archive of all attachments
+   * @returns a promise of the HTTP operation
+   */
+  Message.prototype.downloadAttachmentsArchive = function() {
     var options;
 
     options = { filename: l('attachments') + "-" + this.uid + ".zip" };
