@@ -6,8 +6,8 @@
   /**
    * @ngInject
    */
-  MessageEditorController.$inject = ['$scope', '$window', '$stateParams', '$mdConstant', '$mdUtil', '$mdDialog', '$mdToast', 'FileUploader', 'stateParent', 'stateAccount', 'stateMessage', 'onCompletePromise', 'encodeUriFilter', '$timeout', 'sgFocus', 'Dialog', 'AddressBook', 'Card', 'Preferences'];
-  function MessageEditorController($scope, $window, $stateParams, $mdConstant, $mdUtil, $mdDialog, $mdToast, FileUploader, stateParent, stateAccount, stateMessage, onCompletePromise, encodeUriFilter, $timeout, focus, Dialog, AddressBook, Card, Preferences) {
+  MessageEditorController.$inject = ['$scope', '$window', '$stateParams', '$mdConstant', '$mdUtil', '$mdDialog', '$mdToast', 'FileUploader', 'stateParent', 'stateAccount', 'stateMessage', 'onCompletePromise', 'encodeUriFilter', '$timeout', 'sgConstant', 'sgFocus', 'Dialog', 'AddressBook', 'Card', 'Preferences'];
+  function MessageEditorController($scope, $window, $stateParams, $mdConstant, $mdUtil, $mdDialog, $mdToast, FileUploader, stateParent, stateAccount, stateMessage, onCompletePromise, encodeUriFilter, $timeout, sgConstant, focus, Dialog, AddressBook, Card, Preferences) {
     var vm = this;
 
     this.$onInit = function() {
@@ -17,7 +17,6 @@
       this.autosave = null;
       this.autosaveDrafts = autosaveDrafts;
       this.cancel = cancel;
-      this.contactFilter = contactFilter;
       this.isFullscreen = false;
       this.hideBcc = (stateMessage.editable.bcc.length === 0);
       this.hideCc = (stateMessage.editable.cc.length === 0);
@@ -31,7 +30,6 @@
         $mdConstant.KEY_CODE.COMMA,
         $mdConstant.KEY_CODE.SEMICOLON
       ];
-      this.removeAttachment = removeAttachment;
       this.sendState = false;
       this.toggleFullscreen = toggleFullscreen;
       this.firstFocus = true;
@@ -152,7 +150,7 @@
             $mdToast.simple()
               .textContent(l('Error while uploading the file \"%{0}\":', item.file.name) +
                        ' ' + (response.message? l(response.message) : ''))
-              .position('top right')
+              .position(sgConstant.toastPosition)
               .action(l('OK'))
               .hideDelay(false));
           this.removeFromQueue(item);
@@ -184,11 +182,14 @@
         }
     }
 
-    function removeAttachment(item, id) {
+    this.removeAttachment = function (item, id) {
+      var _this = this;
       if (item.isUploading)
         vm.uploader.cancelItem(item);
       else {
-        vm.message.$deleteAttachment(item.file.name);
+        vm.message.$deleteAttachment(item.file.name).then(function() {
+          _this.save({toast: false});
+        });
         item.remove();
       }
       // Hack to allow adding the same file again
@@ -196,7 +197,7 @@
       var element = $window.document.getElementById(id);
       if (element)
         angular.element(element).prop('value', null);
-    }
+    };
 
     function cancel() {
       if (vm.autosave)
@@ -205,12 +206,21 @@
       if (vm.message.isNew && vm.message.attachmentAttrs)
         vm.message.$mailbox.$deleteMessages([vm.message]);
 
-      $mdDialog.cancel();
+      $mdDialog.hide();
     }
 
-    this.save = function () {
+    // Fix for https://www.sogo.nu/bugs/view.php?id=4666
+    this.ignoreReturn = function ($event) {
+      if ($event.keyCode == 13) {
+        $event.stopPropagation();
+        $event.preventDefault();
+        return false;
+      }
+    };
+
+    this.save = function (options) {
       var ctrls = $parentControllers();
-      this.message.$save().then(function(data) {
+      this.message.$save().then(function() {
         vm.message.$rawSource = null;
         if (ctrls.draftMailboxCtrl) {
           // We're saving a draft from a popup window.
@@ -222,11 +232,13 @@
             }
           });
         }
-        $mdToast.show(
-          $mdToast.simple()
-            .textContent(l('Your email has been saved'))
-            .position('top right')
-            .hideDelay(3000));
+        if (!options || options.toast) {
+          $mdToast.show(
+            $mdToast.simple()
+              .textContent(l('Your email has been saved'))
+              .position(sgConstant.toastPosition)
+              .hideDelay(3000));
+        }
       });
     };
 
@@ -256,7 +268,7 @@
         $mdToast.show(
           $mdToast.simple()
             .textContent(l('Your email has been sent'))
-            .position('top right')
+            .position(sgConstant.toastPosition)
             .hideDelay(3000));
 
         // Let the user see the succesfull message before closing the dialog
@@ -273,8 +285,8 @@
       vm.isFullscreen = !vm.isFullscreen;
     }
 
-    function contactFilter($query) {
-      return AddressBook.$filterAll($query).then(function(cards) {
+    this.contactFilter = function ($query) {
+      return AddressBook.$filterAll($query, [], {priority: 'gcs'}).then(function(cards) {
         // Divide the matching cards by email addresses so the user can select
         // the recipient address of her choice
         var explodedCards = [];
@@ -288,7 +300,7 @@
           return card.$$fullname + ' ' + card.$$email;
         });
       });
-    }
+    };
 
     this.addRecipient = function (contact, field) {
       var recipients, recipient, list, i, address;
@@ -370,7 +382,7 @@
 
       if (this.composeType == "html") {
         nl = '<br />';
-        reNl = '<br ?/>[ \n]?';
+        reNl = '<br ?/>(&nbsp;)?[ \n]?';
         space = '&nbsp;';
       } else {
         nl = '\n';
@@ -397,8 +409,8 @@
 
       if (!previousIdentity && signature.length > 0) {
         // Must place signature at proper place
-        if (!this.isNew() && this.signaturePlacement == 'above') {
-          var quotedMessageIndex = this.message.editable.text.search(new RegExp(reNl + '.+?:( ?' + reNl + '){2}(> |<blockquote type="cite")'));
+        if (!this.isNew() && this.replyPlacement == 'above' && this.signaturePlacement == 'above') {
+          var quotedMessageIndex = this.message.editable.text.search(new RegExp(reNl + '.+?:( ?' + reNl + '){1,2}(> |<blockquote type="cite")'));
           if (quotedMessageIndex >= 0) {
             this.message.editable.text =
               this.message.editable.text.slice(0, quotedMessageIndex) +
